function [X_rec hig_count] = fixed_lag_rec(Y, phi, rho, stdnoise, delta)

%% Initilize

% Dimensions
[m N K] = size(phi);

% Initialize
gamma = ones(N,1);
D = diag(rho); 
count = 0;
hig_count  = 0;
X_rec = zeros(N,K);

% Control parameters
EPSILON     = 1e-8;       % threshold for stopping iteration. 
MAX_ITER    = 5000;       % maximum iterations

%% E-M iterations

%initialize: x(t) --> x(k-t)
x = zeros(N,delta);             % x_{k-del:k-1|k-1}
P_auto = zeros(N,N,delta);      % P_{k-del:k-1|k-1}
P_auto(:,:,1) = eye(N,N);
P_cross = zeros(N,N,delta-1);	% P_{k-del+1,k-del:k,k-1|k-1}
P_last = zeros(N,N,delta);      % P_{k,k-del:k,k-1|k-1}

for k = 1 : K
    count = 0;
    while(1)
        [gammaNew xNew P_autoNew P_crossNew P_lastNew] = ...
                        kalman_lag(D, phi(:,:,k), stdnoise(k), Y(:,k), gamma, x, P_auto, P_cross, P_last);
                      % kalman_lag(A, C, r, y, g, x, P_auto, P_cross, P_last)
        norm(gammaNew - gamma);
        if norm(gammaNew - gamma) < EPSILON
            break; end;
        if count >= MAX_ITER
            break; end;
        gamma = gammaNew;

        count = count+1;
        
    end;
    
    if hig_count < count
        hig_count = count;
    end;
    
    if k > delta
        X_rec(:,k-delta) = xNew(:,delta+1); % x_{k-del|k-1}
    end;    
    x = xNew(:,1:delta);                  % x_{k-del+1:k+1|k-1}
    P_auto = P_autoNew;
    P_cross = P_crossNew;
    P_last = P_lastNew;
end;

X_rec(:,K-delta+1:K) = xNew(:,2:delta+1);