function [gamma x P_auto P_cross P_last] = kalman_lag(A, C, r, y, g, x, P_auto, P_cross, P_last)

% Input paramters
% x(t) = Ax(t-1) + w(t)
% y(t) = Cx(t) + z(t)
% w ~ N(0, Q)
% z ~ N(0, R)

% Dimensions
m = size(y,1);
N = size(A,1);
del = size(x,2);
K = del+1;
%**************************************************************************
%%                               EStep
%**************************************************************************

%Definitions
htranspose = inline('conj(transpose(x))');
R = r*eye(m,m);
Q = (eye(N,N) - A^2)*diag(g);

% x_{k-t|k-1} t = 0,1,...,delta
x(:,2:K) = x;  
x(:,1) = A*x(:,2);      %P_{k|k-1}

% P_{k-t|k-1} t = 0,1,...,delta
P_auto(:,:,2:K) = P_auto;
P_temp = A*P_auto(:,:,2)*A + Q;     % P_{k|k-1}
P_auto(:,:,1) = P_temp;

% P_{k-t,k-t-1|k-1} t = 0,1,...,delta-1
P_cross(:,:,2:del) = P_cross;
P_cross(:,:,1) = P_last(:,:,1);     % P_{k,k-1|k-1}

% P_{k,k-t|k-1} t = 0,1,...,delta
P_last(:,:,2:K) = P_last;
P_last(:,:,1) = P_temp;         % P_{k,k|k-1}

J =  htranspose(C)*inv(C*P_temp*htranspose(C)+R);
v = y - C*x(:,1);


% Updates
for t = 0 : del
    G = htranspose(P_last(:,:,t+1))*J;          % P_{k-t,k|k-1}
    x(:,t+1) = x(:,t+1) + G*v;      % x_{k-t|k}
    P_auto(:,:,t+1)  = P_auto(:,:,t+1) - G*C*P_last(:,:,t+1);    	% P_{k-t|k}
    if t ~= del
        P_cross(:,:,t+1) = (eye(N,N) - G*C)*P_last(:,:,t+2);        % P_{k-t,k-t-1|k}
    end;
    P_last(:,:,t+1)  = A*(eye(N,N) - P_temp*J*C)*P_last(:,:,t+1);   % P_{k+1,k-t|k}
end;

%**************************************************************************
%%                               MStep
%**************************************************************************

gamma = 0;
C_auto_prev = P_auto(:,:,1)+x(:,1)*htranspose(x(:,1));
for t = 1 : del
    C_auto_curr = C_auto_prev;
    C_auto_prev = P_auto(:,:,t+1) + x(:,t+1)*htranspose(x(:,t+1));
    C_cross = P_cross(:,:,t) + x(:,t)*htranspose(x(:,t+1));
    T = C_auto_curr + A*C_auto_prev*A - 2*A*C_cross;
    gamma = gamma +  diag(T);
end;
gamma = 1/K*( inv(eye(N,N)-A^2)*gamma + diag(C_auto_prev) );

P_last(:,:,K) = [];
P_auto(:,:,K) = [];
P_cross(:,:,del) = [];

