function [gamma X P] = kalman_saw(A, C, r, y, g, x, P_auto)

% Input paramters
% x(t) = Ax(t-1) + w(t)
% y(t) = Cx(t) + z(t)
% w ~ N(0, Q)
% z ~ N(0, R)

% Dimensions
htranspose = inline('conj(transpose(x))');
[m K] = size(y);
N = size(A,1);
Q = (eye(N,N) - A^2)*diag(g);
%**************************************************************************
%                               EStep
%**************************************************************************
%% Forward Recursion
x_f = zeros(N, K+1);                % mean x_{t|t}
P_f = zeros(N, N, K+1);             % covarinace P_{t|t}

% initilization
x_f (:,1) = x;
P_f(:,:,1) = P_auto;

for t = 1 : K
    x_temp = A*x_f(:,t);                % x_{t|t-1}
    P_temp = A*P_f(:,:,t)*A'+Q;         % P_{t|t-1}
    K_f = P_temp*htranspose(C(:,:,t))*inv(C(:,:,t)*P_temp*htranspose(C(:,:,t)) + r(t)*eye(m,m));
    x_f(:,t+1) = x_temp + K_f*(y(:,t) - C(:,:,t)*x_temp);  %x_{t|t}
    P_f(:,:,t+1) = P_temp - K_f*C(:,:,t)*P_temp;           %P_{t|t}
end

%% Backward Recursion
x_b = zeros(N, K);              % mean x_{t|K}
P_b = zeros(N, N, K);           % covarinace P_{t|K}
C_b = zeros(N,N,K-1);           % cross covariance P_{t,t-1|K}

% initilization
x_b(:,K) = x_f(:,K+1);                          % x_{K|K}
P_b(:,:,K) = P_f(:,:,K+1);                      % P_{K|K}
if K > 1
    C_b(:,:,K-1) = (eye(N,N) - K_f*C(:,:,K))*A*P_f(:,:,K); % P_{K,K-1|K} 
end;

J_curr = 0;
for t = K : -1 : 2
    P_temp = A*P_f(:,:,t)*A'+Q;         % P_{t|t-1}
    J_prev = J_curr;
    J_curr = P_f(:,:,t)*A'*inv(P_temp);    % J_{t-1}
    x_b(:,t-1) = x_f(:,t) + J_curr*(x_b(:,t) - A*x_f(:,t));               % x_{t-1|K}  
    P_b(:,:,t-1) = P_f(:,:,t) + J_curr*(P_b(:,:,t) - P_temp)*htranspose(J_curr);     % P_{t-1|K}
        
    if t ~= K
        C_b(:,:,t-1) = P_f(:,:,t+1)*htranspose(J_curr)...
                            + J_prev*(C_b(:,:,t) - A*P_f(:,:,t+1))*htranspose(J_curr); % P_{t,t-1|K}
    end;
end;

%**************************************************************************
%%                               MStep
%**************************************************************************

gamma = zeros(60,1);
C_auto_prev = P_b(:,:,K)+x_b(:,K)*htranspose(x_b(:,K));
for t = K : -1 : 2
    C_auto_curr = C_auto_prev;
    C_auto_prev = P_b(:,:,t-1) + x_b(:,t-1)*htranspose(x_b(:,t-1));
    C_cross = C_b(:,:,t-1) + x_b(:,t)*htranspose(x_b(:,t-1));
    T = C_auto_curr + A*C_auto_prev*A - 2*A*C_cross;
    gamma = gamma +  diag(T);
end;
gamma = 1/K*( (eye(N,N)-A^2)^(-1)*gamma + diag(C_auto_prev) );
X = x_b;
P = P_f(:,:,K+1);
