function [X_rec hig_count] = sawtooth_lag_rec(Y, phi, rho, stdnoise, delta)

%% Initilize

% Dimensions
[m N] = size(phi);
K = size(Y,2);
% Initialize
gamma = ones(N,1);
D = diag(rho); 
count = 0;
hig_count = 0;
X_rec = zeros(N,K);

% Control parameters
EPSILON     = 1e-5;       % threshold for stopping iteration. 
MAX_ITER    = 1000;       % maximum iterations

%% E-M iterations

%initialize: x(t) --> x(k-t)
del = max(1,ceil(delta/2));
x_Kal = zeros(N,1);
P_Kal = eye(N,N);

for k = 1 : ceil(K/del)
    count = 0;
    inx = (k-1)*del+1:min(delta+(k-1)*del+1,K);
        
    while(length(inx) ~= 1 || delta == 0)
        [gammaNew xNew PNew ] = ...
                        kalman_saw(D, phi, stdnoise, Y(:,inx), gamma, x_Kal, P_Kal);
                        % kalman_saw(A, C, r, y, g, x, P_auto)

        norm(gammaNew - gamma);
        if norm(gammaNew - gamma) < EPSILON
            break; end;
        if count >= MAX_ITER
            break; end;
        gamma = gammaNew;
        if k == 1
            P_Kal = diag(gammaNew);
        end;
        
        count = count+1;
        
    end;
    
    if hig_count < count
        hig_count = count;
    end;
    if k*del <= K
        X_rec(:,(k-1)*del+1:k*del) = xNew(:,1:del);      % x_{k-del|k-1}
    else
        X_rec(:,(k-1)*del+1:K) = xNew(:,1:K-(k-1)*del);
    end;
    
    if k*del+1 == K && delta ~= 0
        X_rec(:,K) = xNew(:,del+1);
    end;
    x_Kal = xNew(:,length(inx));
    P_Kal = PNew;

end;

X_rec(:,k*del+1:K) = xNew(:,del+1: K-(k-1)*del);